@@ -21,15 +21,16 @@
 #include "Args.hpp"
 #include "ArgsInfo.hpp"
 #include "Config.hpp"
-#include "Digest.hpp"
-#include "File.hpp"
 #include "MiniTrace.hpp"
-#include "NonCopyable.hpp"
+
+#include <util/FileStream.hpp>
+#include <util/NonCopyable.hpp>
 
 #ifdef INODE_CACHE_SUPPORTED
 #  include "InodeCache.hpp"
 #endif
 
+#include <Hash.hpp>
 #include <core/Manifest.hpp>
 #include <storage/Storage.hpp>
 #include <util/TimePoint.hpp>
@@ -43,7 +44,7 @@
 
 class SignalHandler;
 
-class Context : NonCopyable
+class Context : util::NonCopyable
 {
 public:
   Context();
@@ -74,7 +75,7 @@ public:
   util::TimePoint time_of_compilation;
 
   // Files included by the preprocessor and their hashes.
-  std::unordered_map<std::string, Digest> included_files;
+  std::unordered_map<std::string, Hash::Digest> included_files;
 
   // Have we tried and failed to get colored diagnostics?
   bool diagnostics_color_failed = false;
@@ -86,7 +87,7 @@ public:
   util::Bytes cpp_stderr_data;
 
   // Headers (or directories with headers) to ignore in manifest mode.
-  std::vector<std::string> ignore_header_paths;
+  std::vector<std::filesystem::path> ignore_header_paths;
 
   // Storage (fronting local and remote storage backends).
   storage::Storage storage;
@@ -104,7 +105,7 @@ public:
   pid_t compiler_pid = 0;
 
   // Files used by the hash debugging functionality.
-  std::vector<File> hash_debug_files;
+  std::vector<util::FileStream> hash_debug_files;
 
   // Options to ignore for the hash.
   const std::vector<std::string>& ignore_options() const;
