@@ -1,4 +1,4 @@
-// Copyright (C) 2021-2022 Joel Rosdahl and other contributors
+// Copyright (C) 2021-2023 Joel Rosdahl and other contributors
 //
 // See doc/AUTHORS.adoc for a complete list of contributors.
 //
@@ -57,16 +57,29 @@ enum class Statistic {
   could_not_use_modules = 32,
   direct_cache_miss = 33,
   preprocessed_cache_miss = 34,
-  primary_storage_hit = 35,
-  primary_storage_miss = 36,
-  secondary_storage_hit = 37,
-  secondary_storage_miss = 38,
-  secondary_storage_error = 39,
-  secondary_storage_timeout = 40,
+  local_storage_read_hit = 35,
+  local_storage_read_miss = 36,
+  remote_storage_read_hit = 37,
+  remote_storage_read_miss = 38,
+  remote_storage_error = 39,
+  remote_storage_timeout = 40,
   recache = 41,
   unsupported_environment_variable = 42,
+  local_storage_write = 43,
+  local_storage_hit = 44,
+  local_storage_miss = 45,
+  remote_storage_write = 46,
+  remote_storage_hit = 47,
+  remote_storage_miss = 48,
 
-  END
+  // 49-64: files in level 2 subdirs 0-f
+  subdir_files_base = 49,
+
+  // 65-80: size (KiB) in level 2 subdirs 0-f
+  subdir_size_kibibyte_base = 65,
+
+  disabled = 81,
+  END = 82
 };
 
 } // namespace core
