@@ -1,4 +1,4 @@
-// Copyright (C) 2021-2022 Joel Rosdahl and other contributors
+// Copyright (C) 2021-2023 Joel Rosdahl and other contributors
 //
 // See doc/AUTHORS.adoc for a complete list of contributors.
 //
@@ -25,10 +25,9 @@ namespace storage::remote {
 class RedisStorage : public RemoteStorage
 {
 public:
-  std::unique_ptr<Backend>
-  create_backend(const Backend::Params& params) const override;
-
-  void redact_secrets(Backend::Params& params) const override;
+  std::unique_ptr<Backend> create_backend(
+    const Url& url,
+    const std::vector<Backend::Attribute>& attributes) const override;
 };
 
 } // namespace storage::remote
